;	Skeletal CBIOS for first level of CP/M 2.0 alteration
;
; MMBASIC calls
OPTIONS \8080 \REG_SUB 

CPM$COLD$BOOT 	EQU   	20H
CPM$WARM$BOOT 	EQU 	21H
CPM$CONST 		EQU 	22H ; console status
CPM$CONIN 		EQU 	23H ; Console In to A Reg
CPM$CONOUT 		EQU 	24H ; Console Out, from C Reg
CPM$LIST$OUT 	EQU		25H ; Write List Out, from C Reg
CPM$PUNCH$OUT 	EQU		26H ; Write Punch Out, from C Reg
CPM$READER 		EQU		27H ; Reader In to A Reg
CPM$HOME 		EQU		28H ; Disk start
CPM$SEL$DISK 	EQU		29H ; Reg C contains the disk number to select, A = 0, B = 2 etc
CPM$MOVE$TO$TRACK EQU 	2AH	; C Reg contains track
CPM$SETSEC 		EQU		2BH ; C Reg contains sector
CPM$READ$DISK 	EQU		2CH ; read 128 bytes
CPM$WRITE$DISK 	EQU		2DH ; write to disk
CPM$SET$DMA		EQU     2EH ; set DMA address
		

msize	equ	64	;MMBasic cp/m version memory size in kilobytes
;
;	"bias" is address offset from 3400H for memory systems
;	than 16K (referred to as "b" throughout the text).
;
bias	equ	(msize-20)*1024
ccp		equ	3400H+bias	;base of ccp
bdos	equ	ccp+806h	;base of bdos
bios	equ	ccp+1600h	;base of bios
cdisk	equ	0004H	;contains current disk number 0=A,...,15=P
iobyte	equ	0003h	;intel i/o byte address
buff	equ	0FE00h	;buffer address for stack on top of BIOS
;
	org	bios	;origin of this program
nsects	equ	($-ccp)/128	;warm start sector count
;
;	jump vector for individual subroutines
	jmp	boot ; cold start, comes here for MMBasic startup	
wboote:	jmp	wboot		;warm start wboote:
	jmp	const		;console status
	jmp	conin		;console character in
	jmp	conout		;console character out
	jmp	list		;list character out
	jmp	punch		;punch character out
	jmp	reader		;reader character out
	jmp	home		;move head to home position
	jmp	seldsk		;select disk
	jmp	settrk		;set track number
	jmp	setsec		;set sector number
	jmp	setdma		;set dma address
	jmp	read		;read disk
	jmp	write		;write disk
	jmp	listst		;return list status
	jmp	sectran		;sector translate
;
;	fixed data tables for four-drive standard
;	IBM-compatible 8" disks
;	disk parameter header for disk 00
dpbase:	
	dw	trans,0000H ; not using translation table,
	dw	0000H,0000H
	dw	dirbf,dpblk
	dw	chk00,all00
;	disk parameter header for disk 01
	dw	trans,0000H
	dw	0000H,0000H
	dw	dirbf,dpblk
	dw	chk01,all01
;	disk parameter header for disk 02
	dw	trans,0000H
	dw	0000H,0000H
	dw	dirbf,dpblk
	dw	chk02,all02
;	disk parameter header for disk 03
	dw	trans,0000H
	dw	0000H,0000H
	dw	dirbf,dpblk
	dw	chk03,all03
;
;	sector translate vector, for CMM2 use 1:1
trans:	db	1,2,3,4	;sectors 1,2,3,4
	db	5,6,7,8	;sectors 5,6,7,8
	db	9,10,11,12	;sectors 9,10,11,12
	db	13,14,15,16	;sectors 13,14,15,16
	db	17,18,19,20	;sectors 17,18,19,20
	db	21,22,23,24	;sectors 21,22,23,24
	db	25,26		;sectors 25,26
;
dpblk:	;disk parameter block, common to all disks
	dw	26		;sectors per track
	db	3		;BSH, block shift factor, i.e 1024 bytes, shift 128 3 to left = 1024
	db	7		;BLM, block mask, for 1024k
	db	0		;EXM, null mask
	dw	249		;DSM disk size-1, 256,256k, 250 * 1024 (DSM)
	dw	63		;DRM, directory max
	db	192		;alloc 0, 2 blocks of 1024 bytes (64X32 bytes), left 2 bits set hi = C0 = 192
	db	0		;alloc 1
	dw	16		;CKS, make fixed disk,was 16, CKS, check size, 16 * 4 =64 which is the number of directory entries
	dw	0		;track offset, normally 2 but we dont have CP/M system residing on disk so first 2 tracks are free 
;
;	end of fixed tables
;
;	individual subroutines to perform each function
boot:	;The MMBASIC program will load CP/M on a start up
	lxi	sp,buff
	xra	a		;zero in the accum
	sta	iobyte		;clear the iobyte
	sta	cdisk		;select disk zero
	jmp	gocpm		;initialize and go to cp/m
	;lxi	sp,buff
	;xra	a		;zero in the accum
	;sta	iobyte		;clear the iobyte
	;sta	cdisk		;select disk zero
	;jmp	gocpm		;initialize and go to cp/m
;
wboot:	
	call	home		;go to track 00
	lxi	sp,buff
	mvi a, CPM$WARM$BOOT
	MMB 0 ; will return to MMBasic program to reload CCP then return
	;mvi	c,0		;select disk 0
	;call	seldsk
	;call	home		;go to track 00
	;JMP a, CPM$CONIN
;
	;mvi	b,nsects	;b counts # of sectors to load
	;mvi	c,0		;c has the current track number
	;mvi	d,2		;d has the next sector to read
	
;
;	end of load operation, set parameters and go to cp/m
gocpm:
	mvi	a,0c3h	;c3 is a jmp instruction
	sta	0	;for jmp to wboot
	lxi	h,wboote	;wboot entry point
	shld	1	;set address field for jmp at 0
;
	sta	5	;for jmp to bdos
	lxi	h,bdos	;bdos entry point
	shld	6	;address field of jump at 5 to bdos
;
	lxi	b,80h	;default dma address is 80h
	call	setdma
;
	ei		;enable the interrupt system
	lda	cdisk	;get current disk number
	mov	c,a	;send to the ccp
	jmp	ccp	;go to cp/m for further processing
;
;
;	simple i/o handlers (must be filled in by user)
;	in each case, the entry point is provided, with space reserved
;	to insert your own code
;
const:	;console status, return 0ffh if character ready, 00h if not
	mvi a, CPM$CONST
	MMB 0
	ret
;
conin:	;console character into register a, This will hang if no char ready
	mvi a, CPM$CONIN
	MMB 0
	ret
;
conout: ;console character output from register c
	mvi a, CPM$CONOUT
	MMB 0
	ret
;
list:	;list character from register c
	mvi a, CPM$LIST$OUT
	MMB 0
	ret		
;
listst:	;return list status (0 if not ready, 1 if ready)
	xra	a	;0 is always ok to return
	;mvi a, 1
	ret
;
punch:	;Normal CP/M bios punches one character from register c to MMB interface in simulator, 
	; This is modified to write 128 bytes from the default DMA (80h)
	;used to write to CMM2 SD file, MMB returns with A reg set to pReg->nFileExportFlag (1 if exported all OK), 
	;0 if problem or end. This is how the BIOS returns to BDOS
	;lxi h, 080h ; at DMA
	;mvi b, 128 ; count
	;PunchLoop:
	;mov c, m
	mvi a, CPM$PUNCH$OUT
	MMB 0
	ret;
	;cpi 0 ; check MMBasic file open and OK
	;rz ;File end and closed or issue 
	;inx h
	;dcr b
	;rz ; end of 128 bytes
	;jmp PunchLoop
		
;
;
reader: ;read character into register a from reader device
	mvi a, CPM$READER
	MMB 0
	ret
;
;
;	i/o drivers for the disk follow
;	for now, we will simply store the parameters away for use
;	in the read and write subroutines
;
home:	;move to the track 00 position of current drive
;	translate this call into a settrk call with parameter 00
	mvi a, CPM$HOME
	MMB 0
	ret		;we will move to 00 on first read/write
;
seldsk:	;select disk given by register C
	lxi	h,0000h	;error return code
	mov	a,c
	sta	diskno
	cpi	4	;must be between 0 and 3
	rnc		;no carry if 4,5,...
;	disk number is in the proper range
	mvi a, CPM$SEL$DISK
	MMB 0
	cpi 99
	rz ; its an error
;	compute proper disk parameter header address
	lda	diskno
	mov	l,a	;L=disk number 0,1,2,3
	mvi	h,0	;high order zero
	dad	h	;*2
	dad	h	;*4
	dad	h	;*8
	dad	h	;*16 (size of each header)
	lxi	d,dpbase
	dad	d	;HL=.dpbase(diskno*16)
	ret
;
settrk:	;set track given by register c
	mov	a,c
	sta	track
	mvi a, CPM$MOVE$TO$TRACK
	MMB 0
	ret
;
setsec:	;set sector given by register c
	mov	a,c
	sta	sector
	mvi a, CPM$SETSEC
	MMB 0
	ret
;
sectran:
	;translate the sector given by BC using the
	;translate table given by DE
	xchg		;HL=.trans
	dad	b	;HL=.trans(sector)
	mov	l,m	;L = trans(sector)
	mvi	h,0	;HL= trans(sector)
	ret		;with value in HL
;
setdma:	;set dma address given by registers b and c
	mov	l,c	;low order address
	mov	h,b	;high order address
	shld	dmaad	;save the address
	mvi a, CPM$SET$DMA
	MMB 0
	ret
;
read:	;perform read operation 
	mvi a, CPM$READ$DISK
	MMB 0
	ret
;
write:	;perform a write operation
	mvi a, CPM$WRITE$DISK
	MMB 0
	ret
;
;
;	the remainder of the CBIOS is reserved uninitialized
;	data area, and does not need to be a part of the
;	system memory image (the space must be available,
;	however, between "begdat" and "enddat").
;
track:	ds	2	;two bytes for expansion
sector:	ds	2	;two bytes for expansion
dmaad:	ds	2	;direct memory address
diskno:	ds	1	;disk number 0-15
;
;	scratch ram area for BDOS use
begdat	equ	$	;beginning of data area
dirbf:	ds	128	;scratch directory area
all00:	ds	31	;allocation vector 0
all01:	ds	31	;allocation vector 1
all02:	ds	31	;allocation vector 2
all03:	ds	31	;allocation vector 3
 chk00:	ds	16	;check vector 0
 chk01:	ds	16	;check vector 1
 chk02:	ds	16	;check vector 2
 chk03:	ds	16	;check vector 3


;
enddat	equ	$	;end of data area
datsiz	equ	$-begdat;size of data area
	end
